// Slickworm
// Copyright (c) 2002-2003, Michel O'Brien.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  1. Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//  3. Neither the names of the copyright holders nor the names of its
//     contributors may be used to endorse or promote products derived from this
//     software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
// IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#include "flare.h"
#include "gettime.h"

Flare::Flare(TEXTURE_HANDLE fire0, vector3 initial_position, vector3 velocity)
{
    long start_time = GetTime();

    pos = initial_position;
    v = velocity;

    fire = fire0;
    
    
    Snapshot initial, final;
    Particle p;

    initial.size = 0.4f;
    initial.opacity = 0.5f;
    final.size = 0.4f;
    final.opacity = 0.5f;

    p.velocity = v;
    p.initial_position = pos;
    p.maximum_age = 18000;
    p.flags = 0;
    p.mode = SRC_ALPHA_ONE;
    p.texture = fire;
    p.initial = initial;
    p.final = final;

    this->CreateParticle(p, start_time);
    
    initial.size /= 1.2f;
    final.size /= 1.2f;

    p.velocity = v;
    p.initial_position = pos - v * 0.02;
    p.maximum_age = 18000;
    p.flags = 0;
    p.mode = SRC_ALPHA_ONE;
    p.texture = fire;
    p.initial = initial;
    p.final = final;

    this->CreateParticle(p, start_time);
    
    initial.size /= 1.2f;
    final.size /= 1.2f;

    p.velocity = v;
    p.initial_position = pos - v * 0.04;
    p.maximum_age = 18000;
    p.flags = 0;
    p.mode = SRC_ALPHA_ONE;
    p.texture = fire;
    p.initial = initial;
    p.final = final;

    this->CreateParticle(p, start_time);


}

Flare::~Flare()
{


}

void Flare::Update()
{
/*
    long current_time = GetTime();

    if ( (current_time - last_updated) > 50)    // Add a particle to the flare approx. every 50ms
    {
        Snapshot initial, final;

        initial.size = 1.0f;
        initial.opacity = 1.0f;
        final.size = 2.0f;
        final.opacity = 0.0f;

        Particle p;
        p.velocity = v;
        p.initial_position = pos + v * (float(current_time - start_time) / 1000.0f);
        p.maximum_age = 100;
        p.flags = 0;
        p.mode = SRC_ALPHA_ONE;
        p.texture = fire;
        p.initial = initial;
        p.final = final;

        this->CreateParticle(p, current_time);

        // update the last updated time
        last_updated = current_time;
    }
    */

    this->UpdateParticles();
}

void Flare::Render()
{
    this->SubmitParticlesToParticleRenderer();
}
