// Slickworm
// Copyright (c) 2002-2003, Michel O'Brien.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  1. Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//  3. Neither the names of the copyright holders nor the names of its
//     contributors may be used to endorse or promote products derived from this
//     software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
// IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#include "hud_renderer.h"

HUD_Renderer::HUD_Renderer()
{
    vertices[0].u = 0.0f;
    vertices[0].v = 1.0f;
    vertices[0].colour.r = 1.0f;
    vertices[0].colour.g = 1.0f;
    vertices[0].colour.b = 1.0f;

    vertices[1].u = 0.0f;
    vertices[1].v = 0.0f;
    vertices[1].colour.r = 1.0f;
    vertices[1].colour.g = 1.0f;
    vertices[1].colour.b = 1.0f;

    vertices[2].u = 1.0f;
    vertices[2].v = 0.0f;
    vertices[2].colour.r = 1.0f;
    vertices[2].colour.g = 1.0f;
    vertices[2].colour.b = 1.0f;

    vertices[3].u = 1.0f;
    vertices[3].v = 1.0f;
    vertices[3].colour.r = 1.0f;
    vertices[3].colour.g = 1.0f;
    vertices[3].colour.b = 1.0f;

    indices[0] = 0;
    indices[1] = 1;
    indices[2] = 2;
    indices[3] = 2;
    indices[4] = 3;
    indices[5] = 0;
}

HUD_Renderer::~HUD_Renderer()
{
}

void HUD_Renderer::Begin(IGraphicsSystem *igs2)
{
    // Store the pointer to the rendering interface
    igs = igs2;

    // Setup matrices for 2D
    matrix44 identity = IdentityMatrix44();
    igs->SetTransformationMatrix(VIEW_TRANSFORM, identity);
    igs->SetTransformationMatrix(OBJECT_TO_WORLD_TRANSFORM, identity);

    matrix44 perspective_matrix = PerspectiveMatrix44(45.0f, 1.0f, 0.5f, 2000.0f);
    igs->SetTransformationMatrix(PROJECTION_TRANSFORM, perspective_matrix);

    // Default rendering states for 2D text with alpha.
    igs->EnableLighting(false);
    igs->SetTextureWrappingMode(0, CLAMP_TEXTURE);
    igs->SetCullMode(CULL_NONE);
    igs->SetDepthBufferMode(IGNORE_DEPTH);
}

vector3 transform(float x, float y)
{
    x /= 800.0f;
    y /= 600.0f;

    x *= 1.656f;
    y *= 1.656f;

    x -= 0.829f;
    y -= 0.829f;

    return vector3(x, -y, -2.0f);
}

vector3 transform(vector2 v)
{
    return transform(v.x, v.y);
}

void HUD_Renderer::Draw2D(float left_dest, float top_dest, float right_dest, float bottom_dest,
            float left_src, float top_src, float right_src, float bottom_src , COLOUR c)
{
    igs->SetBlendMode(SRC_ALPHA_ONE_MINUS_SRC_ALPHA);

    vertices[0].point = transform(left_dest, bottom_dest);
    vertices[1].point = transform(left_dest, top_dest);
    vertices[2].point = transform(right_dest, top_dest);
    vertices[3].point = transform(right_dest, bottom_dest);

    vertices[0].u = left_src;
    vertices[0].v = bottom_src;

    vertices[1].u = left_src;
    vertices[1].v = top_src;

    vertices[2].u = right_src;
    vertices[2].v = top_src;

    vertices[3].u = right_src;
    vertices[3].v = bottom_src;


    for (int i = 0; i < 4; i++)
    {
        vertices[i].colour.r = c.r;
        vertices[i].colour.g = c.g;
        vertices[i].colour.b = c.b;
        vertices[i].colour.a = c.a;
    }

    igs->DrawIndexedTrianglesUP(TRIANGLE_LIST, VTX_XYZ | VTX_COLOUR | VTX_UV1, (float*)vertices, 4, indices, 6);
}

void HUD_Renderer::Draw2D(vector2 top_left, vector2 top_right, vector2 bottom_right,
    vector2 bottom_left, float left_src, float top_src, float right_src, float bottom_src , COLOUR c)
{
    igs->SetBlendMode(SRC_ALPHA_ONE_MINUS_SRC_ALPHA);

    vertices[0].point = transform(bottom_left);
    vertices[1].point = transform(top_left);
    vertices[2].point = transform(top_right);
    vertices[3].point = transform(bottom_right);

    vertices[0].u = left_src;
    vertices[0].v = bottom_src;

    vertices[1].u = left_src;
    vertices[1].v = top_src;

    vertices[2].u = right_src;
    vertices[2].v = top_src;

    vertices[3].u = right_src;
    vertices[3].v = bottom_src;


    for (int i = 0; i < 4; i++)
    {
        vertices[i].colour.r = c.r;
        vertices[i].colour.g = c.g;
        vertices[i].colour.b = c.b;
        vertices[i].colour.a = c.a;
    }

    igs->DrawIndexedTrianglesUP(TRIANGLE_LIST, VTX_XYZ | VTX_COLOUR | VTX_UV1, (float*)vertices, 4, indices, 6);
}


void HUD_Renderer::End()
{
    // Draw something white and fully opaque off the screen
    COLOUR c;
    c.r = 1.0f;
    c.g = 1.0f;
    c.b = 1.0f;
    c.a = 1.0f;
    
    Draw2D(0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f , c);

    // Restore the normal depth buffer mode
    igs->SetDepthBufferMode(READ_AND_WRITE);

    // Restore the normal blend mode
    igs->SetBlendMode(BLEND_DISABLE);
}











