// Slickworm
// Copyright (c) 2002-2003, Michel O'Brien.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  1. Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//  3. Neither the names of the copyright holders nor the names of its
//     contributors may be used to endorse or promote products derived from this
//     software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
// IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#ifndef KEYBOARD_H
#define KEYBOARD_H

/*!
    \file keyboard.h
    \brief Handles the keyboard.
*/

#include "SDL.h"

#define KEY_A           1
#define KEY_B           2
#define KEY_C           3
#define KEY_D           4
#define KEY_E           5
#define KEY_F           6
#define KEY_G           7
#define KEY_H           8
#define KEY_I           9
#define KEY_J           10
#define KEY_K           11
#define KEY_L           12
#define KEY_M           13
#define KEY_N           14
#define KEY_O           15
#define KEY_P           16
#define KEY_Q           17
#define KEY_R           18
#define KEY_S           19
#define KEY_T           20
#define KEY_U           21
#define KEY_V           22
#define KEY_W           23
#define KEY_X           24
#define KEY_Y           25
#define KEY_Z           26

#define KEY_ESC         30
#define KEY_1           31
#define KEY_2           32
#define KEY_3           33
#define KEY_4           34
#define KEY_5           35
#define KEY_6           36
#define KEY_7           37
#define KEY_8           38
#define KEY_9           39
#define KEY_0           40
#define KEY_MINUS       41  // On main keyboard.
#define KEY_EQUALS      42
#define KEY_BACK        43  // Backspace.

#define KEY_LEFT        100 // Left arrow
#define KEY_RIGHT       101 // Right arrow
#define KEY_UP          102 // Up arrow
#define KEY_DOWN        103 // Down arrow

#define KEY_RETURN      110 // Enter key

#define KEY_PRINT       150
#define KEY_SYSREQ      151
#define KEY_BREAK       152


//! Class that handles input from the keyboard.
class Keyboard
{
public:
    //! Returns a reference to the one and only insatce of this class.
    static Keyboard& Instance()
    {
        static Keyboard Obj;
        return Obj;
    }

    // Returns true if the key corresponding to key is down. Returns false otherwise.
    bool KeyPressed(int keycode);

    // To be called whenever an SDL_KEYUP or SDL_KEYDOWN event is received by the main loop.
    void KeyEventNotify(SDL_KeyboardEvent *key);

private:

    // Constructor is private in this singleton
    Keyboard();

    // Array containing status of all the keys on the keyboard.
    bool key_state[256];
};

#endif // KEYBOARD_H
