// Slickworm
// Copyright (c) 2002-2003, Michel O'Brien.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  1. Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//  3. Neither the names of the copyright holders nor the names of its
//     contributors may be used to endorse or promote products derived from this
//     software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
// IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


// Must document this stuff properly.

#include <stdlib.h>
#include <string>
#include "quadtree_preprocess.h"

float* terrain513;

inline float getheight(int x, int y)
{
    return terrain513[x + y * 513];
}

float geterror(int x, int y, int x1, int y1, int x2, int y2, int x3, int y3, int x4, int y4)
{
    // Given a pyramid shape with ((x1+x3)/2,(y1+y3)/2) as apex and
    // (x1,y1) through (x4,y4) as points on the base this function
    // will return the difference between (x,y) on the pyramid and
    // (x,y) on the heightfield
    int xc = (x1+x3)/2;
    int yc = (y1+y3)/2;

    float realheight = getheight(x,y);

    float h1 = getheight(x1,y1);
    float h2 = getheight(x2,y2);
    float h3 = getheight(x3,y3);
    float h4 = getheight(x4,y4);

    float hc = getheight(xc,yc);

    float pyramidheight=0.0f;

    if ( (x-x1)>(y-y1) )
    {
        // (x,y) is in N or E quadrant

        if ( (x-x4)>(y4-y) )
        {
            // (x,y) is in E quadrant

            float height1 = h2 +( ((h3-h2)*(y-y2))/(y3-y2) );

            float height2 = hc +( ((height1-hc)*(x-xc))/(x2-xc) );

            pyramidheight=height2;
        }
        else
        {
            // (x,y) is in N quadrant
            float height1 = h1 +( ((h2-h1)*(x-x1))/(x2-x1) );

            float height2 = hc +( ((height1-hc)*(yc-y))/(yc-y1) );

            pyramidheight=height2;
        }
    }
    else
    {
        // (x,y) is in S or W quadrant

        if ( (x-x4)>(y4-y) )
        {
            // (x,y) is in S quadrant
            float height1 = h4 +( ((h3-h4)*(x-x4))/(x3-x4) );

            float height2 = hc +( ((height1-hc)*(y-yc))/(y3-yc) );

            pyramidheight=height2;
        }
        else
        {
            // (x,y) is in W quadrant
            float height1 = h1+( ((h4-h1)*(y-y1))/(y4-y1) );

            float height2 = hc+( ((height1-hc)*(xc-x))/(xc-x4) );

            pyramidheight=height2;
        }
    }

    float difference = realheight - pyramidheight;

    // Difference must be positive
    if (difference < 0.0f) difference = -difference;

    return difference;
}

float getmaximumerror(int x1, int y1, int x2, int y2, int x3, int y3, int x4, int y4)
{
    // Given a pyramid shape with ((x1+x3)/2,(y1+y3)/2) as apex and
    // (x1,y1) through (x4,y4) as points on the base this function
    // will return the maximum difference between any point on the
    // pyramid and its corresponding point on the heightfield

    float maximum = 0.0f;

    for (int i = x1; i < x3; i++)
    {
        for (int j= y1; j< y3; j++)
        {
            float error = geterror(i, j, x1, y1, x2, y2, x3, y3, x4, y4);
            if (error > maximum )
            {
                maximum = error;
            }
        }
    }
    return maximum;
}

float getmaximumerror(int x1, int y1, int x2, int y2, int x3, int y3, int x4, int y4,
    int left,int top,int right,int bottom)
{
    // Given a pyramid shape with ((x1+x3)/2,(y1+y3)/2) as apex and
    // (x1,y1) through (x4,y4) as points on the base this function
    // will return the maximum (or 2 * average, depending which section
    // below is commented out) difference between any point on the
    // pyramid (within the rectangle left, top, right, bottom)
    // and its corresponding point on the heightfield

    float maximum = 0.0f;

    for (int i=left;i<right;i++)
    {
        for (int j=top;j<bottom;j++)
        {
            float error = geterror(i, j, x1, y1, x2, y2, x3, y3, x4, y4);
            if (error > maximum )
            {
                maximum = error;
            }
        }
    }
    return (maximum * 4.0f);


/*
    int sum = 0;

    for (int i=left;i<right;i++)
    {
        for (int j=top;j<bottom;j++)
        {
            sum += geterror(i,j,x1,y1,x2,y2,x3,y3,x4,y4);
        }
    }
    sum/=2;

    int average = sum/= ( (right-left)*(bottom-top) );
    return average;
*/

}

// Generates a portion of the quatree lookup table
// corresponding to one of the 256 blocks of terrain
void quadtree_gen_lookup_table(int x,int y, unsigned char *quadtree)
{
    int offset = ((y*16)+x)*342;

    // Get the maximum error for 1 32x32 grid

    float error = getmaximumerror(  x*32,       y*32,
                                    x*32 + 32,  y*32,
                                    x*32 + 32,  y*32 + 32,
                                    x*32,       y*32 + 32);
    *(quadtree+offset+1)=(unsigned char)error;

    int j;

    // Get the maximum errors for 4 16x16 grids
    for (j=0;j<2;j++)
    {
        for (int i=0;i<2;i++)
        {
            error = getmaximumerror(x*32,               y*32,
                                    x*32 + 32,          y*32,
                                    x*32 + 32,          y*32 + 32,
                                    x*32,               y*32 + 32,
                                    x*32 + i*16,        y*32 + j*16,
                                    x*32 + (i + 1)*16,  y*32 + (j + 1)*16);
            *(quadtree+offset+2 + (j*2)+i  )=(unsigned char)error;
        }
    }
    // Get the maximum errors for 16 8x8 grids
    for (j=0;j<4;j++)
    {
        for (int i=0;i<4;i++)
        {
            int i1=i/2;
            int j1=j/2;
            error = getmaximumerror(x*32 + i1*16,       y*32 + j1*16,
                                    x*32 + i1*16 + 16,  y*32 + j1*16,
                                    x*32 + i1*16 + 16,  y*32 + j1*16 + 16,
                                    x*32 + i1*16,       y*32 + j1*16 + 16,
                                    x*32 + i*8,         y*32 + j*8,
                                    x*32 + (i + 1)*8,   y*32 + (j + 1)*8);
            *(quadtree+offset+2+4 + (j*4)+i  )=(unsigned char)error;
        }
    }
    // Get the maximum errors for 64 4x4 grids
    for (j=0;j<8;j++)
    {
        for (int i=0;i<8;i++)
        {
            int i1=i/2;
            int j1=j/2;
            error = getmaximumerror(x*32 + i1*8,        y*32 + j1*8,
                                    x*32 + i1*8 + 8,    y*32 + j1*8,
                                    x*32 + i1*8 + 8,    y*32 + j1*8 + 8,
                                    x*32 + i1*8,        y*32 + j1*8 + 8,
                                    x*32 + i*4,         y*32 + j*4,
                                    x*32 + (i + 1)*4,   y*32 + (j + 1)*4);
            *(quadtree+offset+2+4+16 + (j*8)+i  )=(unsigned char)error;
        }
    }

    // Get the maximum errors for 256 2x2 grids
    for (j=0;j<16;j++)
    {
        for (int i=0;i<16;i++)
        {
            int i1=i/2;
            int j1=j/2;
            error = getmaximumerror(x*32 + i1*4,        y*32 + j1*4,
                                    x*32 + i1*4 + 4,    y*32 + j1*4,
                                    x*32 + i1*4 + 4,    y*32 + j1*4 + 4,
                                    x*32 + i1*4,        y*32 + j1*4 + 4,
                                    x*32 + i*2,         y*32 + j*2,
                                    x*32 + (i + 1)*2,   y*32 + (j + 1)*2);
            *(quadtree+offset+2+4+16+64 + (j*16)+i  )=(unsigned char)error;
        }
    }
}

// Preprocess the quadtree and return a pointer to it.
// terrain is the 513x513 heightfield
unsigned char * quadtree_preprocess(float* terrain)
{
    // The reason for the 342 * 256 * 1 size is as follows:
    // 16 x 16 terrain blocks = 256
    // (1 +) 1 + 4 + 16 + 64 + 256  = 342
    unsigned char* quadtree = new unsigned char[342*256];

    memset(quadtree, 0, 342*256);

    terrain513 = terrain;

    // Build the quadtree lookup table.
    for (int x = 0; x < 16; x++)
    {
        for (int y = 0; y < 16; y++)
        {
            quadtree_gen_lookup_table(x, y, quadtree);
        }
    }

    return quadtree;
}

// Reprocesses the preprocess_data to deal with terrain that has been newly deformed.
// preprocess_data is the 342*256 byte chunk of the same kind that is returned from
// quadtree_preprocess().
// terrain is the 513x513 heightfield of unsigned chars.
// x and y denote the block which is to be recomputed.
void quadtree_reprocess(float* terrain, unsigned char *preprocess_data, int x, int y)
{
    terrain513 = terrain;

    quadtree_gen_lookup_table(x, y, preprocess_data);
}
