// Slickworm
// Copyright (c) 2002-2003, Michel O'Brien.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  1. Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//  3. Neither the names of the copyright holders nor the names of its
//     contributors may be used to endorse or promote products derived from this
//     software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
// IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#include "sea.h"

Sea::Sea(std::string sea_texture_filename, IGraphicsSystem *igs)
{
    // Store the renderer pointer.
    IGS = igs;

    // Create the texture
    sea = IGS->CreateTexture(sea_texture_filename, true, true);
}

Sea::~Sea()
{
    IGS->DeleteTexture(sea);
}

void Sea::Render()
{
    IGS->EnableLighting(false);
    IGS->SetTransformationMatrix(OBJECT_TO_WORLD_TRANSFORM, IdentityMatrix44());

    IGS->SetTexture(0, sea);
    IGS->SetTextureWrappingMode(0, WRAP_TEXTURE);
    IGS->SetCullMode(CULL_CCW);

    vector3 sea_surface[4];

    sea_surface[0] = vector3(-1.0f,  0.0f,  1.0f);
    sea_surface[1] = vector3(-1.0f,  0.0f, -1.0f);
    sea_surface[2] = vector3( 1.0f,  0.0f, -1.0f);
    sea_surface[3] = vector3( 1.0f,  0.0f,  1.0f);
    
    float scale = 256.0f;

    for (int n = 0; n < 4; n++)
    {
        sea_surface[n].x *= scale;
        sea_surface[n].y *= scale;
        sea_surface[n].z *= scale;

        sea_surface[n].x += 256.0f;
        sea_surface[n].z += 256.0f;

        sea_surface[n] -= vector3(2048.0f, -16.1f, 2048.0f);
    }

    VERTEX_XYZ_COLOUR_UV1 points[4];

    points[0].u = 0.0f;
    points[0].v = 0.0f;
    points[0].colour = COLOUR(1.0f, 1.0f, 1.0f, 0.6f);

    points[1].u = 0.0f;
    points[1].v = 16.0f;
    points[1].colour = COLOUR(1.0f, 1.0f, 1.0f, 0.6f);

    points[2].u = 16.0f;
    points[2].v = 16.0f;
    points[2].colour = COLOUR(1.0f, 1.0f, 1.0f, 0.6f);

    points[3].u = 16.0f;
    points[3].v = 0.0f;
    points[3].colour = COLOUR(1.0f, 1.0f, 1.0f, 0.6f);

    unsigned short indices[] = { 0, 1, 2, 2, 3, 0};

    IGS->SetBlendMode(SRC_ALPHA_ONE_MINUS_SRC_ALPHA);
    IGS->SetCullMode(CULL_NONE);

    for (int i = 0; i < 10; i++)
    {
        for (int j = 0; j < 10; j++)
        {
            points[0].point = sea_surface[0] + (float)i * vector3(512.0f, 0.0f, 0.0f) + (float)j * vector3(0.0f, 0.0f, 512.0f);
            points[1].point = sea_surface[1] + (float)i * vector3(512.0f, 0.0f, 0.0f) + (float)j * vector3(0.0f, 0.0f, 512.0f);
            points[2].point = sea_surface[2] + (float)i * vector3(512.0f, 0.0f, 0.0f) + (float)j * vector3(0.0f, 0.0f, 512.0f);
            points[3].point = sea_surface[3] + (float)i * vector3(512.0f, 0.0f, 0.0f) + (float)j * vector3(0.0f, 0.0f, 512.0f);

            IGS->DrawIndexedTrianglesUP(TRIANGLE_LIST, VTX_XYZ | VTX_COLOUR | VTX_UV1, (float*)points, 4, indices, 6);
        }
    }

    IGS->SetCullMode(CULL_CCW);
    IGS->SetBlendMode(BLEND_DISABLE);
}

