// Slickworm
// Copyright (c) 2002-2003, Michel O'Brien.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  1. Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//  3. Neither the names of the copyright holders nor the names of its
//     contributors may be used to endorse or promote products derived from this
//     software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
// IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#include "sky.h"

Sky::Sky(   std::string sky_top_texture_filename,
            std::string sky_west_texture_filename,
            std::string sky_north_texture_filename,
            std::string sky_east_texture_filename,
            std::string sky_south_texture_filename,
            IGraphicsSystem *igs)
{
    // Store the renderer pointer.
    IGS = igs;

    this->top   = IGS->CreateTexture(sky_top_texture_filename, false);
    this->west  = IGS->CreateTexture(sky_west_texture_filename, false);
    this->north = IGS->CreateTexture(sky_north_texture_filename, false);
    this->east  = IGS->CreateTexture(sky_east_texture_filename, false);
    this->south = IGS->CreateTexture(sky_south_texture_filename, false);
}

Sky::~Sky()
{
    IGS->DeleteTexture(top);
    IGS->DeleteTexture(west);
    IGS->DeleteTexture(north);
    IGS->DeleteTexture(east);
    IGS->DeleteTexture(south);
}

void Sky::Render(vector3 camera)
{
    IGS->EnableLighting(false);
    IGS->SetTransformationMatrix(OBJECT_TO_WORLD_TRANSFORM, IdentityMatrix44());
    
    IGS->SetCullMode(CULL_CCW);

    vector3 cube_corners[8];

    cube_corners[0] = vector3(-1.0f,  1.0f,  1.0f);
    cube_corners[1] = vector3(-1.0f,  1.0f, -1.0f);
    cube_corners[2] = vector3( 1.0f,  1.0f, -1.0f);
    cube_corners[3] = vector3( 1.0f,  1.0f,  1.0f);
    cube_corners[4] = vector3(-1.0f, -1.0f,  1.0f);
    cube_corners[5] = vector3(-1.0f, -1.0f, -1.0f);
    cube_corners[6] = vector3( 1.0f, -1.0f, -1.0f);
    cube_corners[7] = vector3( 1.0f, -1.0f,  1.0f);

    float scale = 800.0f;

    for (int i = 0; i < 8; i++)
    {
        cube_corners[i].x *= scale;
        cube_corners[i].y += 0.8f;
        cube_corners[i].y *= scale / 2.0f;
        cube_corners[i].z *= scale;

        cube_corners[i] += camera;
    }

    VERTEX_XYZ_UV1 points[4];

    points[0].u = 0.0f;
    points[0].v = 0.0f;
    points[1].u = 1.0f;
    points[1].v = 0.0f;
    points[2].u = 1.0f;
    points[2].v = 1.0f;
    points[3].u = 0.0f;
    points[3].v = 1.0f;

    unsigned short indices[] = { 0, 1, 2, 2, 3, 0};

    points[0].point = cube_corners[0];
    points[1].point = cube_corners[1];
    points[2].point = cube_corners[5];
    points[3].point = cube_corners[4];

    IGS->SetTexture(0, this->west);
    IGS->SetTextureWrappingMode(0, CLAMP_TEXTURE);
    IGS->DrawIndexedTrianglesUP(TRIANGLE_LIST, VTX_XYZ | VTX_UV1, (float*)points, 4, indices, 6);

    points[0].point = cube_corners[1];
    points[1].point = cube_corners[2];
    points[2].point = cube_corners[6];
    points[3].point = cube_corners[5];

    IGS->SetTexture(0, this->north);
    IGS->SetTextureWrappingMode(0, CLAMP_TEXTURE);
    IGS->DrawIndexedTrianglesUP(TRIANGLE_LIST, VTX_XYZ | VTX_UV1, (float*)points, 4, indices, 6);

    points[0].point = cube_corners[2];
    points[1].point = cube_corners[3];
    points[2].point = cube_corners[7];
    points[3].point = cube_corners[6];

    IGS->SetTexture(0, this->east);
    IGS->SetTextureWrappingMode(0, CLAMP_TEXTURE);
    IGS->DrawIndexedTrianglesUP(TRIANGLE_LIST, VTX_XYZ | VTX_UV1, (float*)points, 4, indices, 6);

    points[0].point = cube_corners[3];
    points[1].point = cube_corners[0];
    points[2].point = cube_corners[4];
    points[3].point = cube_corners[7];

    IGS->SetTexture(0, this->south);
    IGS->SetTextureWrappingMode(0, CLAMP_TEXTURE);
    IGS->DrawIndexedTrianglesUP(TRIANGLE_LIST, VTX_XYZ | VTX_UV1, (float*)points, 4, indices, 6);

    points[0].point = cube_corners[0];
    points[1].point = cube_corners[3];
    points[2].point = cube_corners[2];
    points[3].point = cube_corners[1];

    IGS->SetTexture(0, this->top);
    IGS->SetTextureWrappingMode(0, CLAMP_TEXTURE);
    IGS->DrawIndexedTrianglesUP(TRIANGLE_LIST, VTX_XYZ | VTX_UV1, (float*)points, 4, indices, 6);
}
